package cn.superdesk.data.extract.entity.opCenter;

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import lombok.Builder;
import lombok.Data;
import lombok.EqualsAndHashCode;

import java.io.Serializable;
import java.math.BigDecimal;
import java.util.Date;

/**
 * <p>
 * 运营中心-订单表
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Data
@Builder(toBuilder = true)
@EqualsAndHashCode(callSuper = false)
@TableName("tb_operation_center_order")
public class OperationCenterOrder implements Serializable {

    private static final long serialVersionUID = 1L;

    /**
     * 自增主键
     */
    @TableId(value = "id", type = IdType.AUTO)
    private Long id;

    /**
     * 订单编号
     */
    @TableField("order_sn")
    private String orderSn;

    /**
     * 业务类型id 1-订单 2-售后单
     */
    @TableField("order_biz_type_id")
    private Integer orderBizTypeId;

    /**
     * 业务方订单id
     */
    @TableField("biz_order_id")
    private String bizOrderId;

    /**
     * 业务方订单所属的父订单编号，若未拆单，该字段为空
     */
    @TableField("parent_order_sn")
    private String parentOrderSn;

    /**
     * 业务方订单所属的父订单id，若未拆单，该字段为空。
     */
    @TableField("parent_order_id")
    private String parentOrderId;

    /**
     * 运营中心定义：1-原始订单，2-子订单
     */
    @TableField("order_data_type_id")
    private Integer orderDataTypeId;

    /**
     * 第三方订单号 业务方与第三方的订单编号：京东子单号、超级前台供应链订单编号、蜂助手订单编号、海威、友宝等
     */
    @TableField("third_party_order_sn")
    private String thirdPartyOrderSn;

    /**
     * 业务平台编码 ①FLSC：福利商城②FWZC：服务之窗
     */
    @TableField("biz_platform_code")
    private String bizPlatformCode;

    /**
     * 业务平台名称 ①FLSC：福利商城②FWZC：服务之窗
     */
    @TableField("biz_platform_name")
    private String bizPlatformName;

    /**
     * 订单类型编码1）FLSC：福利商城（2）FZS：蜂助手
     */
    @TableField("order_type_code")
    private String orderTypeCode;

    /**
     * 订单类型名称1）FLSC：福利商城（2）FZS：蜂助手
     */
    @TableField("order_type_name")
    private String orderTypeName;

    /**
     * 订单业务来源id
     */
    @TableField("biz_source_id")
    private Integer bizSourceId;

    /**
     * 订单业务来源名称
     */
    @TableField("biz_source_name")
    private String bizSourceName;

    /**
     * 订单业务终端id
     */
    @TableField("biz_terminal_id")
    private Integer bizTerminalId;

    /**
     * 订单业务终端编码
     */
    @TableField("biz_terminal_code")
    private String bizTerminalCode;

    /**
     * 订单业务终端名称
     */
    @TableField("biz_terminal_name")
    private String bizTerminalName;

    /**
     * 订单时间，订单-下单时间 售后单-售后创建时间
     */
    @TableField("order_create_time")
    private Date orderCreateTime;

    /**
     * 订单支付/退款时间
     */
    @TableField("trading_time")
    private Date tradingTime;

    /**
     * 订单发货时间
     */
    @TableField("send_time")
    private Date sendTime;

    /**
     * 订单完成时间 订单/售后在业务方订单最终状态（已完成/已驳回/已取消/已作废等）的完成时间，若该订单未完成前为空；
     */
    @TableField("finish_time")
    private Date finishTime;

    /**
     * 运营商id
     */
    @TableField("operator_id")
    private String operatorId;

    /**
     * 运营商名称
     */
    @TableField("operator_name")
    private String operatorName;

    /**
     * 项目id
     */
    @TableField("org_id")
    private String orgId;

    /**
     * 项目名称
     */
    @TableField("org_name")
    private String orgName;

    /**
     * 企业id
     */
    @TableField("enterprise_id")
    private String enterpriseId;

    /**
     * 企业名称
     */
    @TableField("enterprise_name")
    private String enterpriseName;

    /**
     * 下单人id
     */
    @TableField("buyer_id")
    private String buyerId;

    /**
     * 下单人姓名
     */
    @TableField("buyer_name")
    private String buyerName;

    /**
     * 下单人部门id （1）订单：所属用户的所属部门id；（2）售后：此字段无数据；
     */
    @TableField("buyer_dept_id")
    private String buyerDeptId;

    /**
     * 下单人部门名称 （1）订单：所属用户的所属部门名称；（2）售后：此字段无数据；
     */
    @TableField("buyer_dept_name")
    private String buyerDeptName;

    /**
     * 下单人手机号
     */
    @TableField("buyer_mobile")
    private String buyerMobile;

    /**
     * 供应商id 售后订单即与原订单所属一致。
     */
    @TableField("supplier_id")
    private String supplierId;

    /**
     * 供应商名称 售后订单即与原订单所属一致。
     */
    @TableField("supplier_name")
    private String supplierName;

    /**
     * 订单商品金额（不含运费）（1）订单：订单所有商品销售总价（折扣前）的金额总和，不含运费；
（2）售后（退货退款/退款）：取该订单所有退货退款或退款的所有商品销售总价（折扣前）的金额总和，不含运费，此值若业务方为正数，需转换为负数；
（3）售后（换货/维修）：金额为0.00；
     */
    @TableField("order_goods_amount")
    private BigDecimal orderGoodsAmount;

    /**
     * 订单协议金额 （1）订单：取该订单所有商品协议价（折扣后）的金额总和，含运费；
（2）售后（退货退款/退款）：取该订单所有退货退款或退款的所有商品协议总价（折扣后）的金额总和，含运费，此值若业务方为正数，需转换为负数；
（3）售后（换货/维修）：金额为0.00；
（4）目前除福利商城“湘江商管”运营商存在协议价，其他运营商可直接取用户实付金额；
     */
    @TableField("order_negotiated_amount")
    private BigDecimal orderNegotiatedAmount;

    /**
     * 订单金额（实付金额） （1）订单：取该订单用户实际支付的金额；
（2）售后（退货退款/退款）：取该订单实际退款给用户的金额，此值若业务方为正数，需转换为负数；
（3）售后（换货/维修）：金额为0.00；
     */
    @TableField("order_price")
    private BigDecimal orderPrice;

    /**
     * 订单结算金额 目前可数据库配置实现此字段数据转换，目前有两种方式取值，以订单（售后）数据所属运营商id与该运营商的订单结算金额取值进行取值：按订单实付金额取值、按订单协议金额取值
（1）订单（售后）的订单结算金额：若该订单（售后）的订单所属运营商id取值为“按订单实付金额取值”，即默认取该数据的“订单实付金额”数据；
（2）订单（售后）的订单结算金额：若该订单（售后）的订单所属运营商id取值为“按订单协议金额取值”，即默认取该数据的“订单协议金额”数据；
（3）目前福利商城的订单（售后）所属运营商为“湘江商管”，即按“订单协议金额”取值，福利商城“湘江商管”运营商id值生产和测试环境可找仕勇提供；
     */
    @TableField("order_settlement_amount")
    private BigDecimal orderSettlementAmount;

    /**
     * 订单成本金额 （1）订单：该订单供应商所有商品的成本金额总和，含运费；
（2）售后（退货退款/退款）：该订单供应商所有退货退款或退款商品的成本金额总和，含运费，此值若业务方为正数，需转换为负数；
（2）售后（换货/维修）：金额为0.00；
     */
    @TableField("order_cost_price")
    private BigDecimal orderCostPrice;

    /**
     * 运费（用户承担用费）（1）运费（用户承担运费）：用户支付给平台的运费；
（2）若金额为空时，需入库时为0.00；
     */
    @TableField("transport_amount")
    private BigDecimal transportAmount;

    /**
     * 商户承担运费 （1）商户承担运费：平台支付给供应商的运费；
（2）若金额为空时，需入库时为0.00；
     */
    @TableField("merchant_transport_amount")
    private BigDecimal merchantTransportAmount;

    /**
     * 优惠金额 （1）订单：取该订单所有优惠项的金额总和；
（2）售后（退货退款/退款）：取该售后订单剔除到优惠金额总和，此值若业务方为正数，需转换为负数；
（3）售后（换货/维修）：金额为0.00；
（4）若优惠金额为空时，需入库时为0.00；
     */
    @TableField("discount_amount")
    private BigDecimal discountAmount;

    /**
     * 优惠券id 多个优惠券时，以“,”隔开存储
     */
    @TableField("coupon_id")
    private String couponId;

    /**
     * 优惠描述 业务方自定义内容：优惠券名称或活动名称，由业务方自定义。
     */
    @TableField("discount_desc")
    private String discountDesc;

    /**
     * 平台售后状态id 运营中心定义：1-未售后，2-售后中，3-售后完成，4-售后取消；
·未售后：指该订单不存在售后订单时的状态；
·售后中：指该订单所关联的售后订单的完成状态存在部分“未完成”状态；
·售后完成：指该订单所关联的售后订单的完成状态均为“已完成”状态；
·售后取消：指该订单所关联的售后订单的完成状态均为“已取消/已驳回”；
（1）该平台售后状态在订单或售后的订单入库时，写入/更新状态值；
（2）订单：指该订单是否存在售后订单的售后状态，初始默认为“未售后”；
（3）售后：指该售后订单的状态“售后中/售后完成/售后取消”，不会出现“未售后”；
     */
    @TableField("platform_aftersale_status_id")
    private Integer platformAftersaleStatusId;

    /**
     * 平台售后类型id 运营中心定义：-1-无，1-换货，2-维修，3-退货退款，4-退款
（1）订单：该字段的值一直为空，由于订单可能存在多个售后订单；
（2）售后：该售后订单的当前售后类型，售后订单不可为空；
     */
    @TableField("platform_aftersale_type_id")
    private Integer platformAftersaleTypeId;

    /**
     * 平台售后类型
     */
    @TableField("platform_aftersale_type")
    private String platformAftersaleType;

    /**
     * 业务订单支付方式id 业务方该订单的支付方式id，非订单流水的支付方式，比如福利商城：1：余额支付，11：后台支付，21：微信支付，24：组合支付，25：云闪付，26：饭卡支付，31：华侨城酒店积分支付，40：餐卡支付，41：餐卡组合支付；
     */
    @TableField("biz_order_pay_type_id")
    private String bizOrderPayTypeId;

    /**
     * 业务订单支付方式
     */
    @TableField("biz_order_pay_type")
    private String bizOrderPayType;

    /**
     * 平台支付方式id （1）根据运营中心管理后台-系统管理-支付方式配置“平台支付方式”与“业务平台+订单类型+业务支付方式”映射关系进行转换入库；
（2）若无映射关系，即平台支付方式为空；
     */
    @TableField("platform_pay_type_id")
    private Long platformPayTypeId;

    /**
     * 平台支付方式
     */
    @TableField("platform_pay_type")
    private String platformPayType;

    /**
     * 订单支付状态id 运营中心定义：1-未支付，2-已支付，3-未退款，4-已退款；
（1）订单：
·未支付：订单未支付的状态；
·已支付：订单已支付的状态；
（2）售后：
·未退款：售后订单未退款的状态；
·已退款：售后订单已退款的状态；
     */
    @TableField("order_pay_status")
    private Integer orderPayStatus;

    /**
     * 业务订单状态id 业务方该订单（售后）的“订单状态id和订单状态名称”内容；
     */
    @TableField("biz_order_status_id")
    private String bizOrderStatusId;

    /**
     * 业务订单状态名称 业务方该订单（售后）的“订单状态id和订单状态名称”内容；
     */
    @TableField("biz_order_status_name")
    private String bizOrderStatusName;

    /**
     * 交易状态id 运营中心定义：1-未完成，2-已完成，3-已取消；
（1）未完成：业务方订单（售后）的订单当前订单状态在业务流程未完成结束的状态；
（2）已完成：业务方订单（售后）的订单当前订单状态在业务流程已完成结束的状态，非“已取消/已驳回/已作废/已删除”的最终状态；
（3）已取消：业务方订单（售后）的订单当前状态在业务流程已结束（已取消/已驳回/已作废/已删除）的状态；
     */
    @TableField("transaction_status_id")
    private Integer transactionStatusId;

    /**
     * 订单备注 业务方可用于记录订单的备注内容和售后退款原因。
     */
    @TableField("order_remark")
    private String orderRemark;

    /**
     * 商家备注
     */
    @TableField("merchant_remark")
    private String merchantRemark;

    /**
     * 收货人 业务方收货信息中的收货人姓名内容
     */
    @TableField("consignee")
    private String consignee;

    /**
     * 收货人电话
     */
    @TableField("consignee_mobile")
    private String consigneeMobile;

    /**
     * 收货地址 业务方收货信息中的收货地址内容，含省+市+区（街道）+详细地址
     */
    @TableField("consignee_address")
    private String consigneeAddress;

    /**
     * 收货地址id
     */
    @TableField("consignee_address_id")
    private String consigneeAddressId;

    /**
     * 收货地址中的“省”
     */
    @TableField("province")
    private String province;

    /**
     * 收货地址中的“市”
     */
    @TableField("city")
    private String city;

    /**
     * 收货地址中的“区”/“县”
     */
    @TableField("county")
    private String county;

    /**
     * 收货地址中的“街道”
     */
    @TableField("street")
    private String street;

    /**
     * 收货地址中的“详细地址”
     */
    @TableField("address_detail")
    private String addressDetail;

    /**
     * 配送方式id
     */
    @TableField("distribution_type_id")
    private String distributionTypeId;

    /**
     * 配送方式名称
     */
    @TableField("distribution_type")
    private String distributionType;

    /**
     * 物流快递编码
     */
    @TableField("express_code")
    private String expressCode;

    /**
     * 物流快递名称
     */
    @TableField("express_name")
    private String expressName;

    /**
     * 物流快递单号
     */
    @TableField("express_tracking_no")
    private String expressTrackingNo;

    /**
     * 退单原单号 （1）订单：为空；
（2）售后：填写原正单的订单编号，售后订单时不可为空；
     */
    @TableField("refund_origin_order_sn")
    private String refundOriginOrderSn;

    /**
     * 创建人id  该数据入库到运营中心的人id，初始一般是系统
     */
    @TableField("create_user_id")
    private String createUserId;

    /**
     * 创建人姓名 该数据入库到运营中心的人，初始一般是系统
     */
    @TableField("create_user_name")
    private String createUserName;

    /**
     * 创建时间
     */
    @TableField("create_time")
    private Date createTime;

    /**
     * 更新人id
     */
    @TableField("update_user_id")
    private String updateUserId;

    /**
     * 更新人姓名
     */
    @TableField("update_user_name")
    private String updateUserName;

    /**
     * 更新时间
     */
    @TableField("update_time")
    private Date updateTime;

    /**
     * 删除状态id 运营中心定义：0-未删除，1-已删除
（1）未删除：指订单在业务系统属于正常数据；
（2）已删除：指订单在业务系统已物理或逻辑删除，目前存在订单拆单以后原父单会在业务系统中逻辑删除，含父单的订单/商品/流水；
（3）订单：根据福利商城订单表“deleted”字段标识该订单是否删除；
（4）售后：售后类订单均默认为“未删除”；
     */
    @TableField("deleted")
    private Integer deleted;


}
