package cn.superdesk.data.extract.job.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.superdesk.data.extract.dto.JobParamDTO;
import cn.superdesk.data.extract.entity.opCenter.OperationDataSyncErrorRecord;
import cn.superdesk.data.extract.entity.flsc.FlscShopGoods;
import cn.superdesk.data.extract.entity.flsc.FlscShopOrder;
import cn.superdesk.data.extract.entity.flsc.FlscShopOrderGoods;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.enums.OpCenterJobSyncTypeEnum;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.FlscShopGoodsMapper;
import cn.superdesk.data.extract.mapper.flsc.FlscShopOrderGoodsMapper;
import cn.superdesk.data.extract.mapper.flsc.FlscShopOrderMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderGoodsMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopOrderGoodsService;
import cn.superdesk.data.extract.service.opCenter.OperationDataSyncErrorRecordService;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * @author xc
 * @description
 * @date 2023/3/28 09:47
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class FlscOrderGoodsSyncJob {
    private final FlscShopOrderGoodsMapper flscShopOrderGoodsMapper;
    private final FlscShopOrderGoodsService flscShopOrderGoodsService;
    private final FlscShopOrderMapper flscShopOrderMapper;
    private final FlscShopGoodsMapper flscShopGoodsMapper;
    private final OperationCenterOrderGoodsMapper operationCenterOrderGoodsMapper;
    private final OperationCenterOrderMapper operationCenterOrderMapper;
    private final OperationDataSyncErrorRecordService operationDataSyncErrorRecordService;


    /**
     * 福利商城正单商品数据同步
     */
    @XxlJob("flscOrderGoodsSyncHandler")
    @Transactional(rollbackFor = Exception.class)
    public void flscOrderGoodsSync() {
        String startTime = DateUtil.now();
        String jobParam = XxlJobHelper.getJobParam();
        log.info("任务参数 : {}", jobParam);
        XxlJobHelper.log("任务参数 : {}", jobParam);
        //解析参数
        JobParamDTO jobParamDTO = null;

        //福利商城订单商品列表
        List<FlscShopOrderGoods> flscShopOrderGoodsList = null;
        //变动数据总数
        Integer total = 0;
        //同步类型
        int syncType;
        //数据查询参数
        Object syncParam;
        if (StringUtils.isNotBlank(jobParam)) {
            try {
                jobParamDTO = JSON.parseObject(jobParam, JobParamDTO.class);
            } catch (Exception e) {
                log.info("任务参数解析异常", e);
                XxlJobHelper.log("任务参数解析异常", e);
                throw new RuntimeException("任务参数解析异常");
            }
            if (jobParamDTO == null || (StringUtils.isBlank(jobParamDTO.getMainParam()) && StringUtils.isBlank(jobParamDTO.getSyncTime()))) {
                log.info("任务参数解析失败 参数内容: {}", jobParam);
                XxlJobHelper.log("任务参数解析失败 参数内容: {}", jobParam);
                throw new RuntimeException("任务参数解析失败");
            }

            String syncTime = jobParamDTO.getSyncTime();
            String mainParam = jobParamDTO.getMainParam();
            if (StringUtils.isNotBlank(syncTime)) {
                //按照指定时间同步
                try {
                    syncParam = DateUtil.parse(syncTime);
                } catch (Exception e) {
                    log.info("任务参数解析异常, 参数内容: {}", jobParam);
                    XxlJobHelper.log("任务参数解析异常, 参数内容 : {}", jobParam);
                    throw new RuntimeException("任务参数解析异常");
                }
                //数据变化量
                total = flscShopOrderGoodsMapper.selectCount(
                        Wrappers.<FlscShopOrderGoods>lambdaQuery()
                                .ge(FlscShopOrderGoods::getSyncTime, syncParam)
                );
                syncType = OpCenterJobSyncTypeEnum.按照指定时间同步.getCode();

            } else {
                //数据变化量
                total = flscShopOrderGoodsMapper.selectCount(
                        Wrappers.<FlscShopOrderGoods>lambdaQuery()
                                .eq(FlscShopOrderGoods::getId, mainParam)
                );
                syncParam = mainParam;
                syncType = OpCenterJobSyncTypeEnum.按照唯一标识同步.getCode();
            }
        }else {
            //获取十五分钟前的时间
            Date fifteenMinutesAgo = DateUtil.offsetMinute(DateUtil.date(), -15);
            total = flscShopOrderGoodsMapper.selectCount(
                    Wrappers.<FlscShopOrderGoods>lambdaQuery()
                            .ge(FlscShopOrderGoods::getSyncTime, fifteenMinutesAgo)
            );
            syncParam = fifteenMinutesAgo;
            syncType = OpCenterJobSyncTypeEnum.默认同步.getCode();
        }

        int errorOrderGoodsCount = 0;
        int batchSize = 1000;

        if (total == 0) {
            //没有数据变动
            log.info("福利商城订单商品同步 无数据变化");
            XxlJobHelper.log("福利商城订单商品同步 无数据变化");
            return;
        } else if (total <= batchSize) {
            //数据变动量小于1000条
            flscShopOrderGoodsList = getOrderGoodsListBySyncType(syncType, syncParam);
            batchHandle(flscShopOrderGoodsList, errorOrderGoodsCount);
        } else {
            //数据变动量大于1000条
            int batch = total / batchSize;
            int remainder = total % batchSize;
            int times = remainder == 0 ? batch : batch + 1;
            int offset = 0;
            for (int i = 0; i < times; i++) {
                flscShopOrderGoodsList = getOrderGoodsListBySyncTypeAndForBatch(syncType, syncParam, offset, batchSize);
                batchHandle(flscShopOrderGoodsList, errorOrderGoodsCount);
                flscShopOrderGoodsList = null;
                offset += batchSize;
            }
        }

        log.info("福利商城订单商品同步结束, 同步类型: {}, 同步参数: {}, 开始时间: {}, 结束时间: {}, 同步数据量: {}, 同步失败数据量: {}",
                OpCenterJobSyncTypeEnum.getDescByCode(syncType), jobParam, startTime, DateUtil.now(), total, errorOrderGoodsCount);
    }


    /**
     * 批量处理
     * @param flscShopOrderGoodsList
     * @param errorOrderGoodsCount
     */
    private void batchHandle(List<FlscShopOrderGoods> flscShopOrderGoodsList, int errorOrderGoodsCount) {
        //福利商城订单商品待新增列表
        List<OperationCenterOrderGoods> operationCenterOrderGoodsInsertList = new ArrayList<>();
        //福利商城订单商品待修改列表
        List<OperationCenterOrderGoods> operationCenterOrderGoodsUpdateList = new ArrayList<>();

        if (CollectionUtil.isNotEmpty(flscShopOrderGoodsList)) {
            for (FlscShopOrderGoods flscShopOrderGoods : flscShopOrderGoodsList) {
                //构造福利商城订单商品数据
                /**
                 * 归属订单
                 */
                FlscShopOrder flscShopOrder = flscShopOrderMapper.selectById(flscShopOrderGoods.getOrderid());
                if (flscShopOrder == null || flscShopOrder.getId() == null) {
                    //异常情况记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(flscShopOrderGoods.getId()))
                                        .orderGoodsId(String.valueOf(flscShopOrderGoods.getId()))
                                        .syncTime(new Date())
                                        .errMsg("订单商品对应订单在业务系统表中不存在")
                                        .retentionData(JSON.toJSONString(flscShopOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("订单商品 id : {}, 关联订单id : {} 不存在", flscShopOrderGoods.getId(), flscShopOrderGoods.getOrderid());
                    continue;
                }
                OperationCenterOrder operationCenterOrder = operationCenterOrderMapper.selectOne(Wrappers.<OperationCenterOrder>lambdaQuery()
                        .eq(OperationCenterOrder::getOrderSn, flscShopOrder.getOrdersn())
                        .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                        .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                        .eq(OperationCenterOrder::getOrderBizTypeId, OpCenterOrderBizTypeEnum.订单.getCode()));
                if (operationCenterOrder == null || operationCenterOrder.getId() == null) {
                    //异常情况记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(flscShopOrderGoods.getId()))
                                        .orderSn(flscShopOrder.getOrdersn())
                                        .orderGoodsId(String.valueOf(flscShopOrderGoods.getId()))
                                        .syncTime(new Date())
                                        .errMsg("订单商品对应订单在运营中心表中不存在")
                                        .retentionData(JSON.toJSONString(flscShopOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("订单商品 id : {}, 关联订单id : {} 运营中心订单不存在", flscShopOrderGoods.getId(), flscShopOrderGoods.getOrderid());
                    continue;
                }
                /**
                 * 关联的商品 shop_goods
                 */
                FlscShopGoods goods = flscShopGoodsMapper.selectById(flscShopOrderGoods.getGoodsid());
                if (goods == null || goods.getId() == null) {
                    //异常情况记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(flscShopOrderGoods.getId()))
                                        .orderSn(flscShopOrder.getOrdersn())
                                        .orderGoodsId(String.valueOf(flscShopOrderGoods.getId()))
                                        .syncTime(new Date())
                                        .errMsg("订单商品对应商品在业务系统表中不存在")
                                        .retentionData(JSON.toJSONString(flscShopOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("订单商品 id : {}, 关联商品id : {} 不存在", flscShopOrderGoods.getId(), flscShopOrderGoods.getGoodsid());
                    continue;
                }
                OperationCenterOrderGoods operationCenterOrderGoods = flscShopOrderGoodsService.buildOpCenterOrderGoodsFlscOrderGoods(flscShopOrderGoods, flscShopOrder, operationCenterOrder, goods);
                //查询运营中心订单商品是否存在
                OperationCenterOrderGoods operationCenterOrderGoodsDB = operationCenterOrderGoodsMapper.selectOne(
                        Wrappers.<OperationCenterOrderGoods>lambdaQuery()
                                .eq(OperationCenterOrderGoods::getOrderSn, operationCenterOrder.getOrderSn())
                                .eq(OperationCenterOrderGoods::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrderGoods::getOrderTypeCode, OpCenterOrderTypeEnum.福利商城.getCode())
                                .eq(OperationCenterOrderGoods::getOrderGoodsId, String.valueOf(flscShopOrderGoods.getId()))
                                .eq(OperationCenterOrderGoods::getBizTypeId, OpCenterOrderBizTypeEnum.订单.getCode())
                );

                if (operationCenterOrderGoodsDB == null || operationCenterOrderGoodsDB.getId() == null) {
                    //新增
                    operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                            .createUserId("0")//创建人
                            .createUserName("增量定时任务同步")
                            .createTime(new Date())
                            .build();
                    operationCenterOrderGoodsInsertList.add(operationCenterOrderGoods);
                } else {
                    //修改
                    operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                            .id(operationCenterOrderGoodsDB.getId())
                            .updateUserId("0")//修改人
                            .updateUserName("增量定时任务同步")
                            .updateTime(new Date())
                            .build();
                    operationCenterOrderGoodsUpdateList.add(operationCenterOrderGoods);
                }
            }
        }


        if (CollectionUtil.isNotEmpty(operationCenterOrderGoodsInsertList)) {
            //循环插入福利商城订单商品
            for (OperationCenterOrderGoods operationCenterOrderGoods : operationCenterOrderGoodsInsertList) {
                try {
                    operationCenterOrderGoodsMapper.insert(operationCenterOrderGoods);
                } catch (Exception e) {
                    //异常情况记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterOrderGoods.getOrderSn())
                                        .orderGoodsId(operationCenterOrderGoods.getOrderGoodsId())
                                        .syncTime(new Date())
                                        .errMsg("订单商品新增失败：" + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("订单商品 id : {}, 插入运营中心订单商品失败", operationCenterOrderGoods.getOrderGoodsId());
                    errorOrderGoodsCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterOrderGoodsUpdateList)) {
            //循环修改福利商城订单商品
            for (OperationCenterOrderGoods operationCenterOrderGoods : operationCenterOrderGoodsUpdateList) {
                try {
                    operationCenterOrderGoodsMapper.updateById(operationCenterOrderGoods);
                } catch (Exception e) {
                    //异常情况记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterOrderGoods.getOrderSn())
                                        .orderGoodsId(operationCenterOrderGoods.getOrderGoodsId())
                                        .syncTime(new Date())
                                        .errMsg("订单商品修改失败：" + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("订单商品 id : {}, 修改运营中心订单商品失败", operationCenterOrderGoods.getOrderGoodsId());
                    errorOrderGoodsCount++;
                }
            }
        }
    }

    /**
     * 根据同步类型获取订单商品列表
     * @param syncType
     * @param syncParam
     * @return
     */
    private List<FlscShopOrderGoods> getOrderGoodsListBySyncType(int syncType, Object syncParam) {
        switch(syncType) {
            case 2:
                //按照唯一标识同步
                return flscShopOrderGoodsMapper.selectList(
                        Wrappers.<FlscShopOrderGoods>lambdaQuery()
                                .eq(FlscShopOrderGoods::getId, syncParam)
                );

            default:
                //默认同步 和 指定时间同步 通用
                return flscShopOrderGoodsMapper.selectList(
                        Wrappers.<FlscShopOrderGoods>lambdaQuery()
                                .ge(FlscShopOrderGoods::getSyncTime, syncParam)
                );
        }
    }

    /**
     * 根据同步类型获取订单商品列表 & 批量
     * @param syncType
     * @param syncParam
     * @param offset
     * @param batchSize
     * @return
     */
    private List<FlscShopOrderGoods> getOrderGoodsListBySyncTypeAndForBatch(int syncType, Object syncParam, int offset, int batchSize) {
        switch(syncType) {
            case 2:
                //按照唯一标识同步
                return flscShopOrderGoodsMapper.selectList(
                        Wrappers.<FlscShopOrderGoods>lambdaQuery()
                                .eq(FlscShopOrderGoods::getId, syncParam)
                                .last("limit " + offset + "," + batchSize)
                );

            default:
                //默认同步 和 指定时间同步 通用
                return flscShopOrderGoodsMapper.selectList(
                        Wrappers.<FlscShopOrderGoods>lambdaQuery()
                                .ge(FlscShopOrderGoods::getSyncTime, syncParam)
                                .last("limit " + offset + "," + batchSize)
                );
        }
    }

}
