package cn.superdesk.data.extract.job.flsc;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.superdesk.data.extract.dto.JobParamDTO;
import cn.superdesk.data.extract.entity.flsc.FlscShopThirdOrderPay;
import cn.superdesk.data.extract.entity.flsc.FlscShopThirdOrderRefund;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.entity.opCenter.OperationDataSyncErrorRecord;
import cn.superdesk.data.extract.enums.OpCenterJobSyncTypeEnum;
import cn.superdesk.data.extract.enums.flsc.FlscThirdAfterSaleStatusEnum;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.FlscShopThirdOrderPayMapper;
import cn.superdesk.data.extract.mapper.flsc.FlscShopThirdOrderRefundMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderGoodsMapper;
import cn.superdesk.data.extract.mapper.opCenter.OperationCenterOrderMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopThirdOrderPayService;
import cn.superdesk.data.extract.service.flsc.FlscShopThirdOrderRefundService;
import cn.superdesk.data.extract.service.opCenter.OperationDataSyncErrorRecordService;
import com.alibaba.fastjson2.JSON;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.xxl.job.core.context.XxlJobHelper;
import com.xxl.job.core.handler.annotation.XxlJob;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author xc
 * @description
 * @date 2023/3/24 16:08
 */
@Component
@Slf4j
@RequiredArgsConstructor
public class FlscThirdOrderRefundSyncJob {

    private final FlscShopThirdOrderRefundMapper flscShopThirdOrderRefundMapper;
    private final FlscShopThirdOrderPayMapper flscShopThirdOrderPayMapper;
    private final FlscShopThirdOrderPayService flscShopThirdOrderPayService;
    private final FlscShopThirdOrderRefundService flscShopThirdOrderRefundService;
    private final OperationCenterOrderMapper operationCenterOrderMapper;
    private final OperationCenterOrderGoodsMapper operationCenterOrderGoodsMapper;
    private final OperationDataSyncErrorRecordService operationDataSyncErrorRecordService;


    @XxlJob(("flscThirdOrderRefundSyncHandler"))
    @Transactional(rollbackFor = Exception.class)
    public void flscThirdOrderRefundSync() {
        String startTime = DateUtil.now();
        String jobParam = XxlJobHelper.getJobParam();
        log.info("任务参数 : {}", jobParam);
        XxlJobHelper.log("任务参数 : {}", jobParam);
        //解析参数
        JobParamDTO jobParamDTO = null;

        //福利商城第三方（蜂助手）退单列表
        List<FlscShopThirdOrderRefund> flscShopThirdOrderRefunds = null;
        //变动数据总数
        Integer total = 0;
        //同步类型
        int syncType;
        //数据查询参数
        Object syncParam;
        if (StringUtils.isNotBlank(jobParam)) {
            try {
                jobParamDTO = JSON.parseObject(jobParam, JobParamDTO.class);
            } catch (Exception e) {
                log.info("任务参数解析异常", e);
                XxlJobHelper.log("任务参数解析异常", e);
                throw new RuntimeException("任务参数解析异常");
            }
            if (jobParamDTO == null || (StringUtils.isBlank(jobParamDTO.getMainParam()) && StringUtils.isBlank(jobParamDTO.getSyncTime()))) {
                log.info("任务参数解析失败 参数内容: {}", jobParam);
                XxlJobHelper.log("任务参数解析失败 参数内容: {}", jobParam);
                throw new RuntimeException("任务参数解析失败");
            }

            String syncTime = jobParamDTO.getSyncTime();
            String mainParam = jobParamDTO.getMainParam();
            if (StringUtils.isNotBlank(syncTime)) {
                //按照指定时间同步
                try {
                    syncParam = DateUtil.parse(syncTime, "yyyy-MM-dd HH:mm:ss");
                } catch (Exception e) {
                    log.info("日期参数解析失败 任务失败 参数内容: {}", jobParam);
                    XxlJobHelper.log("日期参数解析失败 任务失败 参数内容: {}", jobParam);
                    throw new RuntimeException("日期参数解析失败");
                }
                log.info("按照指定时间同步 参数 : {}", syncTime);

                //数据变动总量
                total = flscShopThirdOrderRefundMapper.selectCount(
                        Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                .ge(FlscShopThirdOrderRefund::getSyncTime, syncParam)
                                .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                syncType = OpCenterJobSyncTypeEnum.按照指定时间同步.getCode();
            } else {
                //数据变动总量
                total = flscShopThirdOrderRefundMapper.selectCount(
                        Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                .eq(FlscShopThirdOrderRefund::getRefundno, mainParam)
                                .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                syncParam = mainParam;
                syncType = OpCenterJobSyncTypeEnum.按照唯一标识同步.getCode();
            }
        } else {
            //获取十五分钟前的Date信息
            DateTime fifteenMinutesAgo = DateUtil.offsetMinute(new Date(), -15);
            //数据变动总量
            total = flscShopThirdOrderRefundMapper.selectCount(
                    Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                            .ge(FlscShopThirdOrderRefund::getSyncTime, fifteenMinutesAgo)
                            .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
            );
            syncParam = fifteenMinutesAgo;
            syncType = OpCenterJobSyncTypeEnum.默认同步.getCode();
        }

        int errorOrderCount = 0;
        int errorOrderGoodsCount = 0;
        int batchSize = 1000;

        if (total == 0) {
            log.info("蜂助手退单同步 无数据变动，任务结束");
            XxlJobHelper.log("蜂助手退单同步 无数据变动，任务结束");
            return;
        } else if (total <= 1000) {
            //数据变化量小于1000条
            flscShopThirdOrderRefunds = getOrderListBySyncType(syncType, syncParam);
            batchHandle(flscShopThirdOrderRefunds, errorOrderCount, errorOrderGoodsCount);
        } else {
            //变动数据量大于1000,分批次处理
            int batch = total / batchSize;
            int remainder = total % batchSize;
            int times = remainder == 0 ? batch : batch + 1;
            int offset = 0;
            for (int i = 0; i < times; i++) {
                flscShopThirdOrderRefunds = getOrderListBySyncTypeAndForBatch(syncType, syncParam, offset, batchSize);
                batchHandle(flscShopThirdOrderRefunds, errorOrderCount, errorOrderGoodsCount);
                flscShopThirdOrderRefunds = null;
                offset += batchSize;
            }
        }
        log.info("蜂助手退单同步执行成功，同步类型：{}, 同步参数: {}, 开始时间:{}， 结束时间: {} 本次同步订单数量：{}, 订单总失败记录数: {}, 订单商品总失败记录数: {}",
                OpCenterJobSyncTypeEnum.getDescByCode(syncType), jobParam, startTime, DateUtil.now(), flscShopThirdOrderRefunds.size(), errorOrderCount, errorOrderGoodsCount);
    }

    /**
     * 批量处理
     *
     * @param flscShopThirdOrderRefunds
     * @param errorOrderCount
     * @param errorOrderGoodsCount
     */
    private void batchHandle(List<FlscShopThirdOrderRefund> flscShopThirdOrderRefunds, int errorOrderCount, int errorOrderGoodsCount) {
        //第三方退单待新增列表
        List<OperationCenterOrder> operationCenterRefundOrderInsertList = new ArrayList<>();
        //第三方退单待修改列表
        List<OperationCenterOrder> operationCenterRefundOrderUpdateList = new ArrayList<>();
        //第三方退单商品待新增列表
        List<OperationCenterOrderGoods> operationCenterRefundOrderGoodsInsertList = new ArrayList<>();
        //第三方退单商品待修改列表
        List<OperationCenterOrderGoods> operationCenterRefundOrderGoodsUpdateList = new ArrayList<>();

        if (CollectionUtil.isNotEmpty(flscShopThirdOrderRefunds)) {
            for (FlscShopThirdOrderRefund flscShopThirdOrderRefund : flscShopThirdOrderRefunds) {
                //对应订单id < 0为福利商城手动维护作废的退单，不需要同步
                if (flscShopThirdOrderRefund.getOrderid() < 0) {
                    continue;
                }
                FlscShopThirdOrderPay flscShopThirdOrderPay = flscShopThirdOrderPayMapper.selectById(flscShopThirdOrderRefund.getOrderid());
                //根据福利商城第三方退单组装运营中心退单
                OperationCenterOrder operationCenterOrder = flscShopThirdOrderRefundService.buildOpCenterRefundOrderFromFlscThirdOrder(flscShopThirdOrderRefund, flscShopThirdOrderPay);
                //根据福利商城第三方退单组装运营中心退单商品
                OperationCenterOrderGoods operationCenterOrderGoods = flscShopThirdOrderPayService.buildOpCenterRefundOrderGoodsFromFlscThirdOrder(flscShopThirdOrderRefund, flscShopThirdOrderPay);

                //查询运营中心退单是否存在
                OperationCenterOrder operationCenterOrderDB = operationCenterOrderMapper.selectOne(
                        Wrappers.<OperationCenterOrder>lambdaQuery()
                                .eq(OperationCenterOrder::getOrderSn, operationCenterOrder.getOrderSn())
                                .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                if (operationCenterOrderDB == null || operationCenterOrderDB.getId() == null) {
                    //运营中心退单不存在，新增
                    operationCenterOrder = operationCenterOrder.toBuilder()
                            .createUserId("0")
                            .createUserName("增量定时任务同步")
                            .createTime(new Date())
                            .build();
                    operationCenterRefundOrderInsertList.add(operationCenterOrder);
                } else {
                    //运营中心退单存在，修改
                    operationCenterOrder = operationCenterOrder.toBuilder()
                            .id(operationCenterOrderDB.getId())
                            .updateUserId("0")
                            .updateUserName("增量定时任务同步")
                            .updateTime(new Date())
                            .build();
                    operationCenterRefundOrderUpdateList.add(operationCenterOrder);
                }

                //查询运营中心退单商品是否存在
                OperationCenterOrderGoods operationCenterOrderGoodsDB = operationCenterOrderGoodsMapper.selectOne(
                        Wrappers.<OperationCenterOrderGoods>lambdaQuery()
                                .eq(OperationCenterOrderGoods::getOrderSn, operationCenterOrder.getOrderSn())
                                .eq(OperationCenterOrderGoods::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrderGoods::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
                if (operationCenterOrderGoodsDB == null || operationCenterOrderGoodsDB.getId() == null) {
                    //运营中心订单商品不存在，新增
                    operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                            .createUserId("0")
                            .createUserName("增量定时任务同步")
                            .createTime(new Date())
                            .build();
                    operationCenterRefundOrderGoodsInsertList.add(operationCenterOrderGoods);
                } else {
                    //运营中心订单商品存在，修改
                    operationCenterOrderGoods = operationCenterOrderGoods.toBuilder()
                            .id(operationCenterOrderGoodsDB.getId())
                            .updateTime(new Date())
                            .updateUserId("0")
                            .updateUserName("增量定时任务同步")
                            .build();
                    operationCenterRefundOrderGoodsUpdateList.add(operationCenterOrderGoods);
                }
            }
        }


        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderInsertList)) {
            //循环新增运营中心退单
            for (OperationCenterOrder operationCenterOrder : operationCenterRefundOrderInsertList) {
                try {
                    operationCenterOrderMapper.insert(operationCenterOrder);
                    /**
                     * 反写正单状态
                     */
                    //蜂助手正单
                    FlscShopThirdOrderPay flscShopThirdOrderPay = flscShopThirdOrderPayMapper.selectOne(
                            Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                    .eq(FlscShopThirdOrderPay::getOrdersn, operationCenterOrder.getRefundOriginOrderSn())
                                    .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                    );
                    //蜂助手正单对应的所有退单列表
                    List<FlscShopThirdOrderRefund> flscShopThirdOrderRefundList = flscShopThirdOrderRefundMapper.selectList(
                            Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                    .eq(FlscShopThirdOrderRefund::getOrderid, flscShopThirdOrderPay.getId())
                                    .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                    );
                    if (CollectionUtil.isNotEmpty(flscShopThirdOrderRefundList)) {
                        //运营中心蜂助手正单
                        OperationCenterOrder operationCenterOrderExist = operationCenterOrderMapper.selectOne(Wrappers.<OperationCenterOrder>lambdaQuery()
                                .eq(OperationCenterOrder::getOrderSn, operationCenterOrder.getRefundOriginOrderSn())
                                .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                        );
                        //正单存在，修改售后状态
                        if (operationCenterOrderExist != null && operationCenterOrderExist.getId() != null) {
                            List<Integer> statusList = flscShopThirdOrderRefundList.stream().map(FlscShopThirdOrderRefund::getStatus).distinct().collect(Collectors.toList());
                            if (CollectionUtil.isNotEmpty(statusList) && statusList.size() > 0) {
                                if ((statusList.size() == 1 && statusList.contains(FlscThirdAfterSaleStatusEnum.已通过.getCode())) ||
                                        (statusList.size() == 2 && statusList.contains(FlscThirdAfterSaleStatusEnum.已通过.getCode()) && statusList.contains(FlscThirdAfterSaleStatusEnum.拒绝.getCode()))) {
                                    //1.全部完成 / 部分完成 部分拒绝
                                    operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后完成.getCode());
                                } else if (statusList.size() == 1 && statusList.contains(FlscThirdAfterSaleStatusEnum.拒绝.getCode())) {
                                    //2.全部拒绝
                                    operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后取消.getCode());
                                } else {
                                    //3.全部售后中
                                    //4.部分售后中
                                    operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后中.getCode());
                                }
                                //反写售后单状态
                                operationCenterOrderExist.setUpdateUserId("0");
                                operationCenterOrderExist.setUpdateUserName("定时任务同步退单反写售后状态");
                                operationCenterOrderMapper.updateById(operationCenterOrderExist);
                            }
                        }
                    }
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrder.getBizOrderId()))
                                        .orderSn(operationCenterOrder.getOrderSn())
                                        .syncTime(new Date())
                                        .errMsg("第三方退单信息新增失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrder))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方退单&退单商品增量同步任务执行失败, 新增运营中心退单失败, 退单信息: {}, 异常信息: {}", operationCenterOrder, JSON.toJSONString(e));
                    errorOrderCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderUpdateList)) {
            //循环修改运营中心退单
            for (OperationCenterOrder operationCenterOrder : operationCenterRefundOrderUpdateList) {
                try {
                    operationCenterOrderMapper.updateById(operationCenterOrder);
                    /**
                     * 反写正单状态
                     */
                    //蜂助手正单
                    FlscShopThirdOrderPay flscShopThirdOrderPay = flscShopThirdOrderPayMapper.selectOne(
                            Wrappers.<FlscShopThirdOrderPay>lambdaQuery()
                                    .eq(FlscShopThirdOrderPay::getOrdersn, operationCenterOrder.getRefundOriginOrderSn())
                                    .eq(FlscShopThirdOrderPay::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                    );
                    //蜂助手正单对应的所有退单列表
                    List<FlscShopThirdOrderRefund> flscShopThirdOrderRefundList = flscShopThirdOrderRefundMapper.selectList(
                            Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                    .eq(FlscShopThirdOrderRefund::getOrderid, flscShopThirdOrderPay.getId())
                                    .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                    );
                    if (CollectionUtil.isNotEmpty(flscShopThirdOrderRefundList)) {
                        //运营中心蜂助手正单
                        OperationCenterOrder operationCenterOrderExist = operationCenterOrderMapper.selectOne(Wrappers.<OperationCenterOrder>lambdaQuery()
                                .eq(OperationCenterOrder::getOrderSn, operationCenterOrder.getRefundOriginOrderSn())
                                .eq(OperationCenterOrder::getBizPlatformCode, OpCenterBizPlatformEnum.福利商城.getCode())
                                .eq(OperationCenterOrder::getOrderTypeCode, OpCenterOrderTypeEnum.卡劵商城.getCode())
                        );
                        //正单存在，修改售后状态
                        if (operationCenterOrderExist != null && operationCenterOrderExist.getId() != null) {
                            List<Integer> statusList = flscShopThirdOrderRefundList.stream().map(FlscShopThirdOrderRefund::getStatus).distinct().collect(Collectors.toList());
                            if (CollectionUtil.isNotEmpty(statusList) && statusList.size() > 0) {
                                if ((statusList.size() == 1 && statusList.contains(FlscThirdAfterSaleStatusEnum.已通过.getCode())) ||
                                        (statusList.size() == 2 && statusList.contains(FlscThirdAfterSaleStatusEnum.已通过.getCode()) && statusList.contains(FlscThirdAfterSaleStatusEnum.拒绝.getCode()))) {
                                    //1.全部完成 / 部分完成 部分拒绝
                                    operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后完成.getCode());
                                } else if (statusList.size() == 1 && statusList.contains(FlscThirdAfterSaleStatusEnum.拒绝.getCode())) {
                                    //2.全部拒绝
                                    operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后取消.getCode());
                                } else {
                                    //3.全部售后中
                                    //4.部分售后中
                                    operationCenterOrderExist.setPlatformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.售后中.getCode());
                                }
                                //反写售后单状态
                                operationCenterOrderExist.setUpdateUserId("0");
                                operationCenterOrderExist.setUpdateUserName("定时任务同步退单反写售后状态");
                                operationCenterOrderMapper.updateById(operationCenterOrderExist);
                            }
                        }
                    }
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrder.getBizOrderId()))
                                        .orderSn(operationCenterOrder.getOrderSn())
                                        .syncTime(new Date())
                                        .errMsg("第三方退单信息修改失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrder))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方退单&退单商品增量同步任务执行失败, 修改运营中心退单失败, 退单信息: {}, 异常信息: {}", operationCenterOrder, JSON.toJSONString(e));
                    errorOrderCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderGoodsInsertList)) {
            //循环新增运营中心退单商品
            for (OperationCenterOrderGoods operationCenterOrderGoods : operationCenterRefundOrderGoodsInsertList) {
                try {
                    operationCenterOrderGoodsMapper.insert(operationCenterOrderGoods);
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterOrderGoods.getOrderSn())
                                        .orderGoodsId(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .syncTime(new Date())
                                        .errMsg("第三方退单商品信息新增失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方退单&退单商品增量同步任务执行失败, 新增运营中心退单商品失败, 退单商品信息: {}", operationCenterOrderGoods);
                    errorOrderGoodsCount++;
                }
            }
        }
        if (CollectionUtil.isNotEmpty(operationCenterRefundOrderGoodsUpdateList)) {
            //循环修改运营中心退单商品
            for (OperationCenterOrderGoods operationCenterOrderGoods : operationCenterRefundOrderGoodsUpdateList) {
                try {
                    operationCenterOrderGoodsMapper.updateById(operationCenterOrderGoods);
                } catch (Exception e) {
                    //异常数据需要记录在表
                    ThreadUtil.execute(() -> {
                        operationDataSyncErrorRecordService.save(
                                OperationDataSyncErrorRecord.builder()
                                        .syncType(OperationDataSyncTypeEnum.订单商品.getName())
                                        .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())
                                        .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())
                                        .mainParam(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .orderSn(operationCenterOrderGoods.getOrderSn())
                                        .orderGoodsId(String.valueOf(operationCenterOrderGoods.getOrderGoodsId()))
                                        .syncTime(new Date())
                                        .errMsg("第三方退单商品信息修改失败: " + JSON.toJSONString(e))
                                        .retentionData(JSON.toJSONString(operationCenterOrderGoods))
                                        .handleStatus(OperationDataSyncErrorHandleStatusEnum.未处理.getCode())
                                        .build()
                        );
                    });
                    log.info("福利商城第三方退单&退单商品增量同步任务执行失败, 修改运营中心退单商品失败, 退单商品信息: {}", operationCenterOrderGoods);
                    errorOrderGoodsCount++;
                }
            }
        }
    }

    /**
     * 根据同步类型获取订单列表
     *
     * @param syncType
     * @param syncParam
     * @return
     */
    private List<FlscShopThirdOrderRefund> getOrderListBySyncType(int syncType, Object syncParam) {
        switch (syncType) {
            case 2:
                //按照唯一标识同步
                return flscShopThirdOrderRefundMapper.selectList(
                        Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                .eq(FlscShopThirdOrderRefund::getRefundno, syncParam)
                                .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
            default:
                //默认 & 按时间同步 通用
                return flscShopThirdOrderRefundMapper.selectList(
                        Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                .ge(FlscShopThirdOrderRefund::getSyncTime, syncParam)
                                .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                );
        }
    }

    /**
     * 根据同步类型获取订单列表 & 批量
     *
     * @param syncType
     * @param syncParam
     * @return
     */
    private List<FlscShopThirdOrderRefund> getOrderListBySyncTypeAndForBatch(int syncType, Object syncParam, int offset, int limit) {
        switch (syncType) {
            case 2:
                //按照唯一标识同步
                return flscShopThirdOrderRefundMapper.selectList(
                        Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                .eq(FlscShopThirdOrderRefund::getRefundno, syncParam)
                                .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                                .last("limit " + offset + "," + limit)
                );
            default:
                //默认 & 按时间同步 通用
                return flscShopThirdOrderRefundMapper.selectList(
                        Wrappers.<FlscShopThirdOrderRefund>lambdaQuery()
                                .ge(FlscShopThirdOrderRefund::getSyncTime, syncParam)
                                .eq(FlscShopThirdOrderRefund::getSourceFrom, OpCenterOrderTypeEnum.卡劵商城.getCode())
                                .last("limit " + offset + "," + limit)
                );
        }
    }
}
