package cn.superdesk.data.extract.service.impl.flsc;

import cn.superdesk.data.extract.entity.flsc.FlscShopOperatorsUser;
import cn.superdesk.data.extract.entity.opCenter.UnifiedOperators;
import cn.superdesk.data.extract.enums.opCenter.OpCenterBizPlatformEnum;
import cn.superdesk.data.extract.mapper.flsc.FlscShopOperatorsUserMapper;
import cn.superdesk.data.extract.service.flsc.FlscShopOperatorsUserService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOperatorsService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class FlscShopOperatorsUserServiceImpl extends ServiceImpl<FlscShopOperatorsUserMapper, FlscShopOperatorsUser> implements FlscShopOperatorsUserService {
    private final FlscShopOperatorsUserMapper flscShopOperatorsUserMapper;
    private final OperationCenterOperatorsService operatorCenterOperatorsService;

    /**
     * 全量处理福利商城运营商
     */
    @Override
    @Async
    @Transactional(rollbackFor = Exception.class)
    public void handleAllOperators() {
        List<FlscShopOperatorsUser> list = flscShopOperatorsUserMapper.selectAll();
        log.info("总记录数 : {}", list.size());

        // 成功插入记录数
        int successCount = 0;

        List<UnifiedOperators> insertList = new ArrayList<>();

        for (FlscShopOperatorsUser flscShopOperatorsUser : list) {

            UnifiedOperators operationCenterOperator = UnifiedOperators.builder()
                    .name(flscShopOperatorsUser.getOperatorsName())// 运营商名称
                    .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())// 业务平台编码
                    .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())// 业务平台名称
                    .bizOperatorId(String.valueOf(flscShopOperatorsUser.getId()))// 业务方运营商id
                    .bizOperatorCode(flscShopOperatorsUser.getOperatorsNo())// 运营商编号
                    .contact(flscShopOperatorsUser.getRealname())// 联系人
                    .mobile(flscShopOperatorsUser.getMobile())// 联系电话
                    .address(flscShopOperatorsUser.getAddress())// 地址
                    .remark(flscShopOperatorsUser.getRemark())// 备注
                    .latitude(flscShopOperatorsUser.getLat())// 经度
                    .longitude(flscShopOperatorsUser.getLng())// 纬度
                    .deleted(0)// 是否删除 福利商城默认0
                    .createTime(new Date())
                    .createUserId(0L)
                    .updateTime(null)
                    .updateUserId(null)

                    .build();

            insertList.add(operationCenterOperator);
        }
        successCount = insertList.size();
        operatorCenterOperatorsService.saveBatch(insertList);
        log.info("=======================全量处理运营商数据完成，总记录数: {}, 成功插入记录数: {}=======================", list.size(), successCount);
    }
}
