package cn.superdesk.data.extract.service.impl.flsc;


import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.superdesk.data.extract.entity.flsc.*;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrder;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterOrderGoods;
import cn.superdesk.data.extract.entity.opCenter.OperationCenterPayment;
import cn.superdesk.data.extract.enums.flsc.FlscOrderPayStatusEnum;
import cn.superdesk.data.extract.enums.flsc.FlscPayTypeEnum;
import cn.superdesk.data.extract.enums.opCenter.*;
import cn.superdesk.data.extract.mapper.flsc.*;
import cn.superdesk.data.extract.service.flsc.FlscShopOrderService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOperatorSettlementStrategyService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderGoodsService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterOrderService;
import cn.superdesk.data.extract.service.opCenter.OperationCenterPaymentBizService;
import cn.superdesk.data.extract.utils.PhpDeserializeUtil;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-17
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class FlscShopOrderServiceImpl extends ServiceImpl<FlscShopOrderMapper, FlscShopOrder> implements FlscShopOrderService {
    private final OperationCenterOrderService operationCenterOrderService;
    private final FlscShopOrderMapper shopOrderMapper;
    private final FlscJdVopOrderSubmitOrderMapper flscJdVopOrderSubmitOrderMapper;
    private final FlscQtyxOrderSubmitOrderMapper flscQtyxOrderSubmitOrderMapper;
    private final FlscShopOperatorsUserMapper flscShopOperatorsUserMapper;
    private final FlscShopEnterpriseUserMapper flscShopEnterpriseUserMapper;
    private final FlscShopMemberMapper flscShopMemberMapper;
    private final FlscShopMerchUserMapper flscShopMerchUserMapper;
    private final FlscShopOrderGoodsMapper flscShopOrderGoodsMapper;
    private final OperationCenterOperatorSettlementStrategyService operationCenterOperatorSettlementStrategyService;
    private final FlscShopCouponMapper flscShopCouponMapper;
    private final OperationCenterPaymentBizService operationCenterPaymentBizService;
    private final FlscShopDispatchMapper flscShopDispatchMapper;
    private final OperationCenterOrderGoodsService operationCenterOrderGoodsService;

    /**
     * 全量处理福利商城正单订单数据
     * 包含 福利商城、前台优选、京东订单
     */
    @Async
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void handleAllOrder() {
        Integer count = shopOrderMapper.selectTotalCount();
        int batchSize = 1000;
        int mod = count % batchSize;
        int time = mod > 0 ? count / batchSize + 1 : count / batchSize;
        log.info("总条数 ===== {}", count);
        log.info("处理批次数 ===== {}", time);

        // 成功插入记录数
        int successCount = 0;
        int transportAmountSuccessCount = 0;

        int offset = 0;
        List<FlscShopOrder> list = null;
        List<OperationCenterOrder> insertList = null; // 运营中心订单
        List<OperationCenterOrderGoods> insertList2 = null; // 运费商品
        List<OperationCenterOrderGoods> insertList3 = null; // 商户承担运费商品
        for (int i = 0; i < time; i++) {
            list = shopOrderMapper.selectByBatch(offset, batchSize);
            insertList = new ArrayList<>();
            insertList2 = new ArrayList<>();
            insertList3 = new ArrayList<>();
            for (FlscShopOrder flscShopOrder : list) {
                /**
                 * 构造运营中心订单
                 */
                OperationCenterOrder opCenterOrder = buildOpCenterOrderFromFlscOrder(flscShopOrder);
                opCenterOrder = opCenterOrder.toBuilder()
                        .createUserId("0")
                        .createUserName("全量处理正单")
                        .createTime(new Date())
                        .build();
                insertList.add(opCenterOrder);

                if (BigDecimal.ZERO.compareTo(flscShopOrder.getDispatchprice()) < 0) {
                    /**
                     * 运费大于0时 需要构造运费商品
                     */
                    OperationCenterOrderGoods opCenterOrderTransportAmountGoods = buildOperationCenterFreightOrderGoods(flscShopOrder, opCenterOrder);
                    opCenterOrderTransportAmountGoods = opCenterOrderTransportAmountGoods.toBuilder()
                            .createUserId("0")// 创建人
                            .createUserName("全量处理福利商城订单商品-运费商品")
                            .createTime(new Date())
                            .build();
                    insertList2.add(opCenterOrderTransportAmountGoods);
                }
                /**
                 * 商户承担运费大于0时 需要构造商户承担运费商品
                 */
                BigDecimal merchantTransportAmount = opCenterOrder.getMerchantTransportAmount();
                if (BigDecimal.ZERO.compareTo(merchantTransportAmount) < 0) {
                    OperationCenterOrderGoods opCenterOrderMerchantTransportAmountGoods = buildOperationCenterMerchantFreightOrderGoods(flscShopOrder, opCenterOrder, merchantTransportAmount);
                    opCenterOrderMerchantTransportAmountGoods = opCenterOrderMerchantTransportAmountGoods.toBuilder()
                            .createUserId("0")// 创建人
                            .createUserName("全量处理福利商城订单商品-商户承担运费商品")
                            .createTime(new Date())
                            .build();
                    insertList3.add(opCenterOrderMerchantTransportAmountGoods);
                }


            }
            successCount += insertList.size();
            operationCenterOrderService.saveBatch(insertList);
            transportAmountSuccessCount += insertList2.size();
            operationCenterOrderGoodsService.saveBatch(insertList2);
            operationCenterOrderGoodsService.saveBatch(insertList3);
            insertList = null;
            insertList2 = null;
            insertList3 = null;
            offset += batchSize;
            list = null;
        }
        log.info("=======================全量处理福利商城正单数据完成,总记录数: {}, 成功插入记录数: {}=======================", count, successCount);
        log.info("=======================全量处理福利商城正单运费数据完成 成功插入记录数: {}=======================", transportAmountSuccessCount);
    }

    /**
     * 根据福利商城订单 构造 运营中心订单
     */
    @Override
    public OperationCenterOrder buildOpCenterOrderFromFlscOrder(FlscShopOrder flscShopOrder) {
        String orderSn = flscShopOrder.getOrdersn();
        Integer parentId = flscShopOrder.getParentid();
        String parentOrderSn = null;
        if (parentId != null && parentId != 0) {
            FlscShopOrder parentOrder = shopOrderMapper.selectById(parentId);
            parentOrderSn = parentOrder.getOrdersn();
        }
        String thirdPartyOrderSn = null;

        Integer merchId = flscShopOrder.getMerchid();
        /**
         * 区分订单是 京东订单/前台优选/普通福利商城订单
         */
        BigDecimal merchantTransportAmount = BigDecimal.ZERO;
        if (ObjectUtil.equals(merchId, 8)) {
            // 京东订单
            FlscJdVopOrderSubmitOrder flscJdVopOrderSubmitOrder = flscJdVopOrderSubmitOrderMapper.selectOne(Wrappers.<FlscJdVopOrderSubmitOrder>lambdaQuery().eq(FlscJdVopOrderSubmitOrder::getOrderId, flscShopOrder.getId()));
            if (flscJdVopOrderSubmitOrder != null) {
                thirdPartyOrderSn = flscJdVopOrderSubmitOrder.getJdVopResultJdorderid();
                /**
                 * 商户承担运费，只有京东订单有
                 */
                merchantTransportAmount = flscJdVopOrderSubmitOrder.getJdVopResultFreight();
            }
        } else if (ObjectUtil.equals(merchId, 146)) {
            // 前台优选订单
            FlscQtyxOrderSubmitOrder flscQtyxOrderSubmitOrder = flscQtyxOrderSubmitOrderMapper.selectOne(Wrappers.<FlscQtyxOrderSubmitOrder>lambdaQuery().eq(FlscQtyxOrderSubmitOrder::getShopordersn, flscShopOrder.getOrdersn()));
            if (flscQtyxOrderSubmitOrder != null) {
                thirdPartyOrderSn = flscQtyxOrderSubmitOrder.getThirdordersn();
            }
        } else {
            // 福利商城 or 智臻
            if (StringUtils.isNotBlank(flscShopOrder.getYxpackageid())) {
                thirdPartyOrderSn = flscShopOrder.getYxpackageid();
            }
        }
        /**
         * 福利商城运营商id
         */
        Integer operatorId = flscShopOrder.getOperatorsId();
        String operatorName = "";
        if (operatorId != null && operatorId != 0) {
            FlscShopOperatorsUser flscShopOperatorsUser = flscShopOperatorsUserMapper.selectById(operatorId);
            if (flscShopOperatorsUser != null) {
                operatorName = flscShopOperatorsUser.getOperatorsName();
            }
        }
        /**
         * 福利商城项目id
         */
        Integer organizationId = flscShopOrder.getMemberOrganizationId();
        String organizationName = "";
        /**
         * 福利商城企业id
         */
        Integer memberEnterpriseId = flscShopOrder.getMemberEnterpriseId();
        if (memberEnterpriseId != null && memberEnterpriseId != 0) {
            FlscShopEnterpriseUser flscShopEnterpriseUser = flscShopEnterpriseUserMapper.selectById(memberEnterpriseId);
            if (flscShopEnterpriseUser != null) {
                organizationName = StringUtils.isBlank(flscShopEnterpriseUser.getOrganName()) ? "" : flscShopEnterpriseUser.getOrganName();
            }
        }
        /**
         * 福利商城用户openid
         */
        String openid = flscShopOrder.getOpenid();
        String buyerName = null;
        String buyerDeptName = null;
        String buyerMobile = null;
        FlscShopMember flscShopMember = flscShopMemberMapper.selectOne(Wrappers.<FlscShopMember>lambdaQuery().eq(FlscShopMember::getOpenid, openid));
        if (flscShopMember != null) {
            buyerName = flscShopMember.getRealname();
            buyerDeptName = flscShopMember.getDepartmentName();
            buyerMobile = flscShopMember.getMobile();
        }
        /**
         * 福利商城供应商id
         */
        String supplierName = "";
        FlscShopMerchUser flscShopMerchUser = flscShopMerchUserMapper.selectById(merchId);
        if (flscShopMerchUser != null) {
            supplierName = flscShopMerchUser.getMerchname();
        }
        /**
         * 根据订单查所有商品 计算相关金额
         */
        // 订单商品金额 不含运费
        BigDecimal orderGoodsAmount = BigDecimal.ZERO;
        // 订单协议金额 含运费
        BigDecimal orderNegotiatedAmount = BigDecimal.ZERO;
        // 订单成本金额 含商户承担运费
        BigDecimal orderCostPrice = BigDecimal.ZERO;
        // 订单实付金额
        BigDecimal orderPrice = flscShopOrder.getPrice();

        List<FlscShopOrderGoods> flscShopOrderGoods = flscShopOrderGoodsMapper.selectList(Wrappers.<FlscShopOrderGoods>lambdaQuery().eq(FlscShopOrderGoods::getOrderid, flscShopOrder.getId()));
        if (CollectionUtil.isNotEmpty(flscShopOrderGoods)) {
            // 订单商品金额 = 商品marketprice * total / marketprice_total
            orderGoodsAmount = flscShopOrderGoods.stream().map(g -> g.getMarketPrice().multiply(new BigDecimal(g.getTotal()))).reduce(BigDecimal.ZERO, BigDecimal::add);
            // 订单协议金额 = 商品协议价 * 商品数量 + 用户承担运费
            orderNegotiatedAmount = flscShopOrderGoods.stream().map(order -> order.getCostpriceReal().multiply(new BigDecimal(order.getTotal()))).reduce(BigDecimal.ZERO, BigDecimal::add);
            orderNegotiatedAmount = orderNegotiatedAmount.add(flscShopOrder.getDispatchprice());
            // 订单成本金额 = 商品成本价 * 商品数量 + 商户承担运费
            orderCostPrice = flscShopOrderGoods.stream().map(order -> order.getCostprice().multiply(new BigDecimal(order.getTotal()))).reduce(BigDecimal.ZERO, BigDecimal::add);
            orderCostPrice = orderCostPrice.add(merchantTransportAmount);
        }
        /**
         * 订单结算金额
         */
        BigDecimal orderSettlementAmount = operationCenterOperatorSettlementStrategyService.getOrderSettlementAmount(operatorId, OpCenterBizPlatformEnum.福利商城.getCode(), orderPrice, orderNegotiatedAmount);

        /**
         * 优惠金额： 包含两部分 叠加
         * category_enterprise_discount 企业折扣优惠
         * couponprice 优惠券优惠
         */
        BigDecimal discountAmount = flscShopOrder.getCategoryEnterpriseDiscount().add(flscShopOrder.getCouponprice());

        /**
         * 优惠券描述
         */
        Integer couponid = flscShopOrder.getCouponid();
        String discountDesc = null;
        if (couponid != null && couponid != 0) {
            FlscShopCoupon flscShopCoupon = flscShopCouponMapper.selectById(couponid);
            if (flscShopCoupon != null) {
                discountDesc = flscShopCoupon.getDesc();
            }
        }

        /**
         * 平台支付方式
         */
        OperationCenterPayment platformPayment = operationCenterPaymentBizService.getPlatformPayment(OpCenterBizPlatformEnum.福利商城.getCode(), flscShopOrder.getPaytype());

        /**
         * 反序列化address获取收货信息
         * { "area": "双流区", "jd_vop_town_code": "71576", "address": "测试", "town": "怡心街道", "city": "成都市", "openid": "wap_user_17_15008155867_30991", "mobile": "15525252525", "label": "家", "jd_vop_county_code": "49324", "uniacid": "17", "realname": "张三", "zipcode": "", "deleted": "0", "jd_vop_province_code": "22", "province": "四川", "jd_vop_area": "22_1930_49324_71576", "jd_vop_city_code": "1930", "id": "9485", "isdefault": "1" }
         */
        String address = flscShopOrder.getAddress();
        HashMap<String, String> deserializedMap = PhpDeserializeUtil.getDeserialized(address);
        String consignee = StringUtils.isBlank(deserializedMap.get("realname")) ? "" : deserializedMap.get("realname");
        String consigneeMobile = StringUtils.isBlank(deserializedMap.get("mobile")) ? "" : deserializedMap.get("mobile");
        String consigneeAddressId = StringUtils.isBlank(deserializedMap.get("id")) ? "" : deserializedMap.get("id");
        String province = StringUtils.isBlank(deserializedMap.get("province")) ? "" : deserializedMap.get("province");
        String city = StringUtils.isBlank(deserializedMap.get("city")) ? "" : deserializedMap.get("city");
        String county = StringUtils.isBlank(deserializedMap.get("area")) ? "" : deserializedMap.get("area");
        String street = StringUtils.isBlank(deserializedMap.get("town")) ? "" : deserializedMap.get("town");
        String addressDetail = StringUtils.isBlank(deserializedMap.get("address")) ? "" : deserializedMap.get("address");

        /**
         * 配送方式
         */
        Integer distributionTypeId = flscShopOrder.getDispatchid();
        String distributionType = null;
        if (distributionTypeId != null && distributionTypeId != 0) {
            FlscShopDispatch flscShopDispatch = flscShopDispatchMapper.selectById(distributionTypeId);
            distributionType = flscShopDispatch.getDispatchname();
        }

        /**
         * 完成时间
         */
        Date finishTime;
        if (flscShopOrder.getCanceltime() != null && flscShopOrder.getCanceltime() > 0) {
            finishTime = new Date(flscShopOrder.getCanceltime() * 1000L);
        } else {
            finishTime = flscShopOrder.getFinishtime() > 0 ? new Date(flscShopOrder.getFinishtime() * 1000L) : null;
        }

        return OperationCenterOrder.builder()
                .orderBizTypeId(OpCenterOrderBizTypeEnum.订单.getCode())// 业务类型id
                .orderSn(orderSn)// 订单编号
                .bizOrderId(String.valueOf(flscShopOrder.getId()))// 订单id
                .parentOrderSn(parentOrderSn)// 父订单编号，通过父订单id查询
                .parentOrderId(parentId == null || parentId == 0 ? null : String.valueOf(parentId))// 父订单id
                .orderDataTypeId(parentId == null || parentId == 0 || orderSn.equalsIgnoreCase(parentOrderSn) ? OpCenterOrderDataTypeEnum.原始订单.getCode() : OpCenterOrderDataTypeEnum.子订单.getCode())// 订单数据类型id
                .thirdPartyOrderSn(thirdPartyOrderSn)// 三方单号
                .bizPlatformCode(OpCenterBizPlatformEnum.福利商城.getCode())// 业务平台编码
                .bizPlatformName(OpCenterBizPlatformEnum.福利商城.getName())// 业务平台名称
                .orderTypeCode(OpCenterOrderTypeEnum.福利商城.getCode())// 订单类型编码
                .orderTypeName(OpCenterOrderTypeEnum.福利商城.getName())// 订单类型名称
                .bizSourceId(OpCenterBizSourceEnum.福利商城.getCode())// 业务来源id
                .bizSourceName(OpCenterBizSourceEnum.福利商城.getDesc())// 业务来源名称
                .bizTerminalId(OpCenterBizTerminalEnum.微信小程序.getCode())// 业务终端id
                .bizTerminalCode(null)// 业务终端编码，福利商城置空
                .bizTerminalName(null)// 业务终端名称，福利商城置空
                .orderCreateTime(new Date(flscShopOrder.getCreatetime() * 1000L))// 订单时间
                .tradingTime(flscShopOrder.getPaytime() > 0 ? new Date(flscShopOrder.getPaytime() * 1000L) : null)// 订单支付时间
                .sendTime(flscShopOrder.getSendtime() > 0 ? new Date(flscShopOrder.getSendtime() * 1000L) : null)// 发货时间
                .finishTime(finishTime)// 完成时间
                .operatorId(String.valueOf(operatorId))// 运营商id（业务方）
                .operatorName(operatorName)// 运营商名称（业务方） 福利商城从ims_superdesk_shop_operators_user表里取
                .orgId(organizationId == null ? null : String.valueOf(organizationId))// 项目id
                .orgName(organizationName)// 项目名称
                .enterpriseId(memberEnterpriseId == null ? null : String.valueOf(memberEnterpriseId))// 企业id
                .enterpriseName(flscShopOrder.getMemberEnterpriseName())// 企业名称
                .buyerId(openid)// 下单人userId 福利商城取openid
                .buyerName(buyerName)// 下单人姓名
                .buyerDeptId(null)// 下单人部门id 福利商城没有
                .buyerDeptName(buyerDeptName)// 下单人部门名称
                .buyerMobile(buyerMobile)// 下单人手机号
                .supplierId(String.valueOf(merchId))// 供应商id
                .supplierName(supplierName)// 供应商名称
                .orderGoodsAmount(orderGoodsAmount)// 订单所有商品销售总价之和，不含运费
                .orderNegotiatedAmount(orderNegotiatedAmount)// 订单协议金额 含用户承担运费
                .orderPrice(orderPrice)// 订单金额（实付金额）
                .orderSettlementAmount(orderSettlementAmount)// 订单结算金额
                .orderCostPrice(orderCostPrice)// 订单成本金额 含商户承担运费
                .transportAmount(flscShopOrder.getDispatchprice())// 运费
                .merchantTransportAmount(merchantTransportAmount)// 商户承担运费 只有京东订单有
                .discountAmount(discountAmount)// 优惠金额
                .couponId(couponid == null ? null : String.valueOf(couponid))// 优惠券id
                .discountDesc(discountDesc)// 优惠券描述
                .platformAftersaleStatusId(OpCenterPlatformAftersaleStatusEnum.未售后.getCode())// 平台售后状态id
                .platformAftersaleTypeId(null)// 平台售后类型id
                .platformAftersaleType(null)// 平台售后类型
                .bizOrderPayTypeId(String.valueOf(flscShopOrder.getPaytype()))// 业务订单支付方式id
                .bizOrderPayType(FlscPayTypeEnum.getNameById(flscShopOrder.getPaytype()))// 业务订单支付方式
                .platformPayTypeId(platformPayment.getId())// 平台支付方式id
                .platformPayType(platformPayment.getName())// 平台支付方式名称
                .orderPayStatus(flscShopOrder.getPaytime() != null && flscShopOrder.getPaytime() > 0 ? OpCenterOrderPayStatusEnum.已支付.getCode() : OpCenterOrderPayStatusEnum.未支付.getCode())// 订单支付状态
                .bizOrderStatusId(String.valueOf(flscShopOrder.getStatus()))// 业务订单状态id
                .bizOrderStatusName(FlscOrderPayStatusEnum.getBizOrderPayStatusDescByBizId(flscShopOrder.getStatus()))// 业务订单状态名称
                .transactionStatusId(FlscOrderPayStatusEnum.getOpTransactionDescByBizId(flscShopOrder.getStatus()))// 交易状态id
                .orderRemark(flscShopOrder.getRemark())// 订单备注
                .merchantRemark(flscShopOrder.getRemarksaler())// 商家备注
                .consignee(consignee)// 收货人
                .consigneeMobile(consigneeMobile)// 收货人电话
                .consigneeAddress(province + city + county + street + addressDetail)
                .consigneeAddressId(consigneeAddressId)// 收货地址id
                .province(province)// 省
                .city(city)// 市
                .county(county)// 区/县
                .street(street)// 街道
                .addressDetail(addressDetail)
                .distributionTypeId(distributionTypeId == null ? null : String.valueOf(distributionTypeId))// 配送方式id
                .distributionType(distributionType)// 配送方式名称
                .expressCode(flscShopOrder.getExpress())// 物流快递编码
                .expressName(flscShopOrder.getExpresscom())// 物流快递名称
                .expressTrackingNo(flscShopOrder.getExpresssn())// 物流快递单号
                .refundOriginOrderSn(null)// 退单原单号 正单为空
                /**
                 * 福利给的条件：deleted = 1 或 isparent = 1 都代表是已拆单的父单
                 */
                .deleted(ObjectUtil.equals(flscShopOrder.getDeleted(), 1) || ObjectUtil.equals(flscShopOrder.getIsparent(), 1) ? 1 : 0)// 是否删除
                .build();
    }

    /**
     * 构建福利商城运费订单商品
     *
     * @param flscShopOrder
     * @param opCenterOrder
     * @return
     */
    @Override
    public OperationCenterOrderGoods buildOperationCenterFreightOrderGoods(FlscShopOrder flscShopOrder, OperationCenterOrder opCenterOrder) {
        return OperationCenterOrderGoods.builder()
                .bizTypeId(OpCenterOrderBizTypeEnum.订单.getCode())// 业务类型id
                .orderGoodsId(OpCenterOrderTypeEnum.福利商城.getCode() + "_" + OpCenterGoodsTypeEnum.运费.getPrefix() + flscShopOrder.getId())// 订单商品id 运费id构造：FLSC_YF订单id
                .orderSn(flscShopOrder.getOrdersn())// 订单编号
                .orderId(String.valueOf(flscShopOrder.getId()))// 订单id
                .parentOrderId(opCenterOrder.getParentOrderSn())// 父订单id，取订单的
                .parentOrderSn(opCenterOrder.getParentOrderSn())// 父订单sn，取订单的
                .thirdPartyOrderSn(opCenterOrder.getThirdPartyOrderSn())// 第三方单号
                .bizPlatformCode(opCenterOrder.getBizPlatformCode())// 业务平台编码
                .bizPlatformName(opCenterOrder.getBizPlatformName())// 业务平台名称
                .orderTypeCode(opCenterOrder.getOrderTypeCode())// 订单类型编码
                .orderTypeName(opCenterOrder.getOrderTypeName())// 订单类型名称
                .orderCreateTime(opCenterOrder.getOrderCreateTime())// 订单时间
                .finishTime(opCenterOrder.getFinishTime())// 完成时间
                .isPackage(OpCenterIsPackageEnum.非套餐.getCode())// 是否套餐
                .packageId(null)// 套餐id
                .packageName(null)// 套餐名称
                .goodsTypeId(OpCenterGoodsTypeEnum.运费.getCode())// 商品类型id
                .goodsId(String.valueOf(OpCenterCustomizeGoodsEnum.运费.getCode()))// 商品id
                .goodsSku(null)// 商品sku
                .goodsName(OpCenterGoodsTypeEnum.运费.getDesc())// 商品名称
                .quantity(1)// 商品数量
                .unit("项")// 商品单位
                .originalUnitPrice(flscShopOrder.getDispatchprice())// 商品原单价
                .originalTotalPrice(flscShopOrder.getDispatchprice())// 商品原总价
                .sellingTotalPrice(flscShopOrder.getDispatchprice())// 商品销售总价
                .sellingUnitPrice(flscShopOrder.getDispatchprice())// 商品销售单价
                .negotiatedTotalPrice(flscShopOrder.getDispatchprice())// 商品协议总价
                .negotiatedUnitPrice(flscShopOrder.getDispatchprice())// 商品协议单价
                .orderGoodsAmount(flscShopOrder.getDispatchprice())// 订单商品总价 20230425新增字段
                .settlementAmount(flscShopOrder.getDispatchprice())// 结算总价
                .settlementUnitPrice(flscShopOrder.getDispatchprice())// 结算单价
                .costUnitPrice(flscShopOrder.getDispatchprice())// 成本单价
                .costTotalPrice(flscShopOrder.getDispatchprice())// 成本总价
                .discountAmount(BigDecimal.ZERO)// 优惠金额 先置空 等业务方给方案
                .platformTaxCode(null)// 平台税务编码
                .platformTaxRate(BigDecimal.ZERO)// 平台税率
                .merchantTaxCode(null)// 商户税务编码
                .merchantTaxRate(BigDecimal.ZERO)// 商户税率
                .isDutyFree(OpCenterIsDutyFreeEnum.不免税.getCode())// 默认不免税
                .operatorId(opCenterOrder.getOperatorId())// 运营商id
                .operatorName(opCenterOrder.getOperatorName())// 运营商名称
                .supplierId(opCenterOrder.getSupplierId())// 供应商id
                .supplierName(opCenterOrder.getSupplierName())// 供应商名称
                .brandId(null)// 品牌id 预留 暂时不管
                .brandName(null)// 品牌名称 预留 暂时不管
                .cateLevel1Id(null)// 商品一级分类id
                .cateLevel1Name(null)// 商品一级分类名称
                .cateLevel2Id(null)// 商品二级分类id
                .cateLevel2Name(null)// 商品二级分类名称
                .cateLevel3Id(null)// 商品三级分类id
                .cateLevel3Name(null)// 商品三级分类名称
                .goodsSpecification(null)// 商品型号/规格
                .otherAttribute(null)// 其他属性
                .refundOriginOrderSn(null)// 退单原正单sn 正单商品为空
                .build();
    }

    /**
     * 构造商户承担运费商品
     *
     * @param flscShopOrder
     * @param opCenterOrder
     * @param merchantTransportAmount
     * @return
     */
    public OperationCenterOrderGoods buildOperationCenterMerchantFreightOrderGoods(FlscShopOrder flscShopOrder, OperationCenterOrder opCenterOrder, BigDecimal merchantTransportAmount) {
        return OperationCenterOrderGoods.builder()
                .bizTypeId(OpCenterOrderBizTypeEnum.订单.getCode())// 业务类型id
                .orderGoodsId(OpCenterOrderTypeEnum.福利商城.getCode() + "_" + OpCenterGoodsTypeEnum.商户承担运费.getPrefix() + flscShopOrder.getId())// 订单商品id 运费id构造：FLSC_SHYF订单id
                .orderSn(flscShopOrder.getOrdersn())// 订单编号
                .orderId(String.valueOf(flscShopOrder.getId()))// 订单id
                .parentOrderId(opCenterOrder.getParentOrderSn())// 父订单id，取订单的
                .parentOrderSn(opCenterOrder.getParentOrderSn())// 父订单sn，取订单的
                .thirdPartyOrderSn(opCenterOrder.getThirdPartyOrderSn())// 第三方单号
                .bizPlatformCode(opCenterOrder.getBizPlatformCode())// 业务平台编码
                .bizPlatformName(opCenterOrder.getBizPlatformName())// 业务平台名称
                .orderTypeCode(opCenterOrder.getOrderTypeCode())// 订单类型编码
                .orderTypeName(opCenterOrder.getOrderTypeName())// 订单类型名称
                .orderCreateTime(opCenterOrder.getOrderCreateTime())// 订单时间
                .finishTime(opCenterOrder.getFinishTime())// 完成时间
                .isPackage(OpCenterIsPackageEnum.非套餐.getCode())// 是否套餐
                .packageId(null)// 套餐id
                .packageName(null)// 套餐名称
                .goodsTypeId(OpCenterGoodsTypeEnum.商户承担运费.getCode())// 商品类型id
                .goodsId(String.valueOf(OpCenterCustomizeGoodsEnum.商户承担运费.getCode()))// 商品id
                .goodsSku(null)// 商品sku
                .goodsName(OpCenterGoodsTypeEnum.商户承担运费.getDesc())// 商品名称
                .quantity(1)// 商品数量
                .unit("项")// 商品单位
                .originalUnitPrice(merchantTransportAmount)// 商品原单价
                .originalTotalPrice(merchantTransportAmount)// 商品原总价
                .sellingTotalPrice(merchantTransportAmount)// 商品销售总价
                .sellingUnitPrice(merchantTransportAmount)// 商品销售单价
                .negotiatedTotalPrice(merchantTransportAmount)// 商品协议总价
                .negotiatedUnitPrice(merchantTransportAmount)// 商品协议单价
                .settlementAmount(merchantTransportAmount)// 结算总价
                .settlementUnitPrice(merchantTransportAmount)// 结算单价
                .costUnitPrice(merchantTransportAmount)// 成本单价
                .costTotalPrice(merchantTransportAmount)// 成本总价
                .discountAmount(BigDecimal.ZERO)// 优惠金额 先置空 等业务方给方案
                .platformTaxCode(null)// 平台税务编码
                .platformTaxRate(BigDecimal.ZERO)// 平台税率
                .merchantTaxCode(null)// 商户税务编码
                .merchantTaxRate(BigDecimal.ZERO)// 商户税率
                .isDutyFree(OpCenterIsDutyFreeEnum.不免税.getCode())// 默认不免税
                .operatorId(opCenterOrder.getOperatorId())// 运营商id
                .operatorName(opCenterOrder.getOperatorName())// 运营商名称
                .supplierId(opCenterOrder.getSupplierId())// 供应商id
                .supplierName(opCenterOrder.getSupplierName())// 供应商名称
                .brandId(null)// 品牌id 预留 暂时不管
                .brandName(null)// 品牌名称 预留 暂时不管
                .cateLevel1Id(null)// 商品一级分类id
                .cateLevel1Name(null)// 商品一级分类名称
                .cateLevel2Id(null)// 商品二级分类id
                .cateLevel2Name(null)// 商品二级分类名称
                .cateLevel3Id(null)// 商品三级分类id
                .cateLevel3Name(null)// 商品三级分类名称
                .goodsSpecification(null)// 商品型号/规格
                .otherAttribute(null)// 其他属性
                .refundOriginOrderSn(null)// 退单原正单sn 正单商品为空
                .build();
    }

}
