package cn.superdesk.data.extract.service.opCenter;

import cn.superdesk.data.extract.entity.opCenter.OperationCenterOperatorSettlementStrategy;
import com.baomidou.mybatisplus.extension.service.IService;

import java.math.BigDecimal;

/**
 * <p>
 * 运营中心-运营商结算策略关系表 服务类
 * </p>
 *
 * @author storyxc
 * @since 2023-03-20
 */
public interface OperationCenterOperatorSettlementStrategyService extends IService<OperationCenterOperatorSettlementStrategy> {

    /**
     * 根据运营商配置的结算策略 - 获取订单结算金额
     *
     * @param operatorId            运营商id （业务方的）
     * @param bizPlatformCode       业务平台编码
     * @param orderPrice            订单实付金额
     * @param orderNegotiatedAmount 订单协议金额
     * @return
     */
    BigDecimal getOrderSettlementAmount(Integer operatorId, String bizPlatformCode, BigDecimal orderPrice, BigDecimal orderNegotiatedAmount);

    /**
     * 根据运营商配置的结算策略 - 获取订单商品结算总价
     *
     * @param operatorId           运营商id 业务方的
     * @param bizPlatformCode      业务平台编码
     * @param sellingTotalPrice    销售总价
     * @param negotiatedTotalPrice 协议总价
     * @return
     */
    BigDecimal getOrderGoodsSettlementAmount(String operatorId, String bizPlatformCode, BigDecimal sellingTotalPrice, BigDecimal negotiatedTotalPrice);

    /**
     * 根据运营商配置的结算策略 - 获取订单商品结算单价
     *
     * @param operatorId          运营商id 业务方的
     * @param bizPlatformCode     业务平台编码
     * @param sellingUnitPrice    销售单价
     * @param negotiatedUnitPrice 协议单价
     * @return
     */
    BigDecimal getOrderGoodsSettlementUnitPrice(String operatorId, String bizPlatformCode, BigDecimal sellingUnitPrice, BigDecimal negotiatedUnitPrice);
}
